unit KA.Data.KADao.SelectIndexButton;

interface

uses
  Borland.Vcl.SysUtils,
  Borland.Vcl.Classes,
  Borland.Vcl.Controls,
  Borland.Vcl.Forms,
  Borland.Vcl.Dialogs,
  Borland.Vcl.StdCtrls,
  Borland.Vcl.DB,
  Borland.Vcl.DBCtrls;

type
  TKADaoSelectIndexButton = class(TButton)
  private
    { Private declarations }
    FDataLink     : TFieldDataLink;
    FBefore       : Boolean;
  protected
    { Protected declarations }
    Function    GetDataSource: TDataSource;
    Procedure   SetDataSource(Value: TDataSource);
    Procedure   Notification(AComponent: TComponent; Operation: TOperation); override;
    Procedure   HandleClickEvent;
    Procedure   DataChange(Sender: TObject);
    Procedure   ActiveChange(Sender: TObject);
    procedure   Loaded; override;
  public
    { Public declarations }
    Constructor Create(AOwner: TComponent);override;
    Destructor  Destroy;override;
    Procedure   Click; override;
  published
    { Published declarations }
    Property    DataSource  : TDataSource Read GetDataSource Write SetDataSource;
    Property    CallBefore  : Boolean     Read FBefore         Write FBefore;
  end;

procedure Register;

implementation
Uses
  KA.Data.KADao.Table,
  KA.Data.KADao.SelectIndexUnit;

Const
  dbOpenTable : Integer = 1;

Constructor TKADaoSelectIndexButton.Create(AOwner: TComponent);
Begin
  inherited Create(AOwner);
  Caption     := 'Select Index';
  FDataLink  := TFieldDataLink.Create;
  FDataLink.OnDataChange    := DataChange;
  FDataLink.OnActiveChange  := ActiveChange;
  FBefore    := True;
End;

Destructor TKADaoSelectIndexButton.Destroy;
Begin
  FDataLink.Free;
  FDataLink := nil;
  Inherited Destroy;
End;

Function TKADaoSelectIndexButton.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

Procedure TKADaoSelectIndexButton.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active)
  And (TKaDaoTable(DataSource.Dataset).TableType=dbOpenTable) Then
      Begin
        Enabled := True;
      End
  Else
      Begin
        Enabled := False;
      End;
end;

Procedure TKADaoSelectIndexButton.ActiveChange(Sender: TObject);
Begin
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active)
  And (TKaDaoTable(DataSource.Dataset).TableType=dbOpenTable) Then
      Begin
        Enabled := True;
      End
  Else
      Begin
        Enabled := False;
      End;
End;

Procedure TKADaoSelectIndexButton.DataChange(Sender: TObject);
Begin
if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active)
  And (TKaDaoTable(DataSource.Dataset).TableType=dbOpenTable) Then
      Begin
        Enabled := True;
      End
  Else
      Begin
        Enabled := False;
      End;
End;

Procedure TKADaoSelectIndexButton.Loaded;
Begin
  Inherited Loaded;
  if  Assigned(DataSource)
  And Assigned(DataSource.Dataset)
  And (DataSource.DataSet is TKaDaoTable)
  And (DataSource.DataSet.Active)
  And (TKaDaoTable(DataSource.Dataset).TableType=dbOpenTable) Then
      Begin
        Enabled := True;
      End
  Else
      Begin
        Enabled := False;
      End;
End;

procedure TKADaoSelectIndexButton.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and (AComponent = DataSource) then DataSource := nil;
end;

Procedure TKADaoSelectIndexButton.HandleClickEvent;
Var
  NewIndex : String;
Begin
 if  Assigned(DataSource)
 And Assigned(DataSource.Dataset)
 And (DataSource.DataSet is TKaDaoTable)
 And (DataSource.DataSet.Active) Then
    Begin
      if TKaDaoTable(DataSource.Dataset).TableType=dbOpenTable Then
         Begin
           NewIndex:=TKaDaoTable(DataSource.Dataset).IndexName;
           Application.CreateForm(TKADaoSelectIndex, KADaoSelectIndex);
           if KADaoSelectIndex.Execute(TKaDaoTable(DataSource.Dataset), NewIndex) Then
              Begin
              End;
           KADaoSelectIndex.Free;
           TKaDaoTable(DataSource.Dataset).IndexName := NewIndex;
         End;
    End;
End;

Procedure TKADaoSelectIndexButton.Click;
Begin
 if FBefore Then inherited Click;
 HandleClickEvent;
 if NOT FBefore Then inherited Click;
End;

procedure Register;
begin
  RegisterComponents('KA Dao Controls', [TKADaoSelectIndexButton]);
end;

end.
